namespace Calcolatrice {

using NUnit.Framework;

[TestFixture]
public class TestCalcolatrice {

    private Calcolatrice calcolatrice;

    [SetUp]
    protected void SetUp() {
        calcolatrice = new Calcolatrice();
    }

    [Test]
    public void TestSomma() {
        calcolatrice.Somma(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 5.4, "Somma 1");
        calcolatrice.Somma(0, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 3.4, "Somma 2");
        calcolatrice.Somma(2, -1.5);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0.5, "Somma 3");
    }

    [Test]
    public void TestSottrai() {
        calcolatrice.Sottrai(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == -1.4, "Sottrai 1");
        calcolatrice.Sottrai(0, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == -3.4, "Sottrai 2");
        calcolatrice.Sottrai(2, -1.5);
        Assert.IsTrue(calcolatrice.GetRisultato() == 3.5, "Sottrai 3");
    }

    [Test]
    public void TestMoltiplica() {
        calcolatrice.Moltiplica(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 6.8, "Moltiplica 1");
        calcolatrice.Moltiplica(0, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0, "Moltiplica 2");
        calcolatrice.Moltiplica(1, -1.5);
        Assert.IsTrue(calcolatrice.GetRisultato() == -1.5, "Moltiplica 3");
    }

    [Test]
    public void TestDividi() {
        calcolatrice.Dividi(3, 1.5);
        Assert.IsTrue(calcolatrice.GetRisultato() == 2, "Dividi 1");
        calcolatrice.Dividi(3, 1);
        Assert.IsTrue(calcolatrice.GetRisultato() == 3, "Dividi 2");
        calcolatrice.Dividi(0, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0, "Dividi 3");
        try {
            calcolatrice.Dividi(2, 0);
            Assert.Fail();
        } catch (System.Exception) {}
    }

    [Test]
    public void TestSommaAlRisultato() {
        calcolatrice.SommaAlRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == 10, "Somma al risultato iniziale");
        calcolatrice.Somma(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 5.4, "Somma iniziale");
        calcolatrice.SommaAlRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == 15.4, "Somma al risultato 1");
        calcolatrice.SommaAlRisultato(0);
        Assert.IsTrue(calcolatrice.GetRisultato() == 15.4, "Somma al risultato 2");
        calcolatrice.SommaAlRisultato(-1.5);
        Assert.IsTrue(calcolatrice.GetRisultato() == 13.9, "Somma al risultato 3");
    }

    [Test]
    public void TestSottraiDalRisultato() {
        calcolatrice.SottraiDalRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == -10, "Sottrai dal risultato iniziale");
        calcolatrice.Somma(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 5.4, "Somma iniziale");
        calcolatrice.SottraiDalRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == -4.6, "Sottrai dal risultato 1");
        calcolatrice.SottraiDalRisultato(0);
        Assert.IsTrue(calcolatrice.GetRisultato() == -4.6, "Sottrai dal risultato 2");
        calcolatrice.SottraiDalRisultato(-1.5);
        Assert.IsTrue(Uguali(calcolatrice.GetRisultato(), -3.1), "Sottrai dal risultato 3");
    }

    [Test]
    public void TestMoltiplicaPerIlRisultato() {
        calcolatrice.MoltiplicaPerIlRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0, "Moltiplica per il risultato iniziale");
        calcolatrice.Somma(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 5.4, "Somma iniziale");
        calcolatrice.MoltiplicaPerIlRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == 54, "Moltiplica per il risultato 1");
        calcolatrice.MoltiplicaPerIlRisultato(1);
        Assert.IsTrue(calcolatrice.GetRisultato() == 54, "Moltiplica per il risultato 2");
        calcolatrice.MoltiplicaPerIlRisultato(0);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0, "Moltiplica per il risultato 3");
    }

    [Test]
    public void TestDividiIlRisultato() {
        calcolatrice.DividiIlRisultato(10);
        Assert.IsTrue(calcolatrice.GetRisultato() == 0, "Dividi il risultato iniziale");
        calcolatrice.Somma(2, 3.4);
        Assert.IsTrue(calcolatrice.GetRisultato() == 5.4, "Somma iniziale");
        calcolatrice.DividiIlRisultato(2);
        Assert.IsTrue(calcolatrice.GetRisultato() == 2.7, "Dividi il risultato 1");
        try {
            calcolatrice.DividiIlRisultato(0);
            Assert.Fail();
        } catch (System.Exception) {}
    }

    private bool Uguali (double a, double b) {
        return (System.Math.Abs(a - b) < 1E-10);
    }

}

}